<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\UsuariosService;
use FM\Traits\TResponse;

class UsuariosController extends FMCrud {
    
    use TResponse;
    protected $dataSourceName = 'fcm_checklistagend';
    protected $UsuariosService;

    public function __construct($crudService, $environment, UsuariosService $UsuariosService) {
        parent::__construct($crudService, $environment);
        $this->UsuariosService = $UsuariosService;
        $this->Environment = $environment;
    }

    // Busca uma lista de usuários, todas as informações
    public function getUsersController(DTO\Request $request, DTO\Response $response) {
        try {
            $data = $this->UsuariosService->getUsersService();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Busca uma lista de usuários, só nome e id
    public function usersListController(DTO\Request $request, DTO\Response $response) {
        $var = $this->UsuariosService->usersListService();
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

    public function createUserController(DTO\Request $request, DTO\Response $response) {

        try {
            $params = $request->getParameters();

            $nmfuncionario = $params['userInfos']['NMFUNCIONARIO'];
            $nrperfilacesso = $params['userInfos']['NRPERFILACESSO'];
            $nrcpffuncionario = $params['userInfos']['NRCPFFUNCIONARIO'];
            $nrrgfuncionario = $params['userInfos']['NRRGFUNCIONARIO'];
            $dsemailfuncionario = $params['userInfos']['DSEMAILFUNCIONARIO'];
            $nrtelfuncionario = $params['userInfos']['NRTELFUNCIONARIO'];
            $cdoperador = $params['userInfos']['CDOPERADOR'];
            $nrsetor = $params['userInfos']['NRSETOR'];

            $data = $this->UsuariosService->createUserService($nmfuncionario, $nrperfilacesso, $nrcpffuncionario, $nrrgfuncionario, $dsemailfuncionario, $nrtelfuncionario, $nrsetor,$cdoperador);
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // Busca as informações de um usuário específico
    public function getUserInfosController(DTO\Request $request, DTO\Response $response) {

        $params = $request->getParameters();

        $nrfuncionario = $params['params']['NRFUNCIONARIO'];

        $var = $this->UsuariosService->getUserInfosService($nrfuncionario);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

    public function updateUserController(DTO\Request $request, DTO\Response $response) {
            
        try {
            $params = $request->getParameters();
            
            $nrfuncionario = $params['params']['NRFUNCIONARIO'];
            $nmfuncionario = $params['params']['NMFUNCIONARIO'];
            $nrperfilacesso = $params['params']['NRPERFILACESSO'];
            $nrcpffuncionario = $params['params']['NRCPFFUNCIONARIO'];
            $nrrgfuncionario = $params['params']['NRRGFUNCIONARIO'];
            $dsemailfuncionario = $params['params']['DSEMAILFUNCIONARIO'];
            $nrtelfuncionario = $params['params']['NRTELFUNCIONARIO'];
            $nrsetor = $params['params']['NRSETOR'];
            $relacfuncperfacess = $params['params']['RELACFUNCPERFACESS'];
            $nrrelacfuncsetor = $params['params']['NRRELACFUNCSETOR'];
            $cdoperador = $params['params']['CDOPERADOR'];

            $data = $this->UsuariosService->updateUserService($nrfuncionario, $nmfuncionario, $nrperfilacesso, $nrcpffuncionario, $nrrgfuncionario, $dsemailfuncionario, $nrtelfuncionario, $nrsetor, $relacfuncperfacess, $nrrelacfuncsetor, $cdoperador);
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteUserController(DTO\Request $request, DTO\Response $response) {
    
        try {
            $data = $this->UsuariosService->deleteUsersService($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOperadorController(DTO\Request $request, DTO\Response $response) {
        $response->addDataSet(new DataSet('operadores', $this->UsuariosService->getOperadorService()));
    }
}
?>