<?php
namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00003000ChecklistFactory {

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    // FUNÇÕES DA TAB DE CHECKLIST

    public function getAtividadesChecklist($parametros) {
        $sql = " SELECT FCM_ITEMCHECKLIST.NRTAREFA, FCM_TAREFA.NMTAREFA 
                    FROM FCM_ITEMCHECKLIST
                    JOIN FCM_TAREFA
                    ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
                    WHERE FCM_ITEMCHECKLIST.NRATIVIDADE IN ({$parametros})
                    AND FCM_ITEMCHECKLIST.IDATIVO = 'S'
                    ORDER BY FCM_TAREFA.NMTAREFA";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosChecklist($parametros) {

        $sql = "SELECT DISTINCT TO_CHAR(FCM_ITEMCHECKLIST.NRESCOPOSTAREFA)AS CODIGO, FCM_ITEMCHECKLIST.NRESCOPOSTAREFA AS CODIGOAUX, FCM_ESCOPOSTAREFA.NMESCOPOSTAREFA AS NOME, 'Escopo' AS TIPO, NULL AS CODIGOPAI, NULL AS CODIGOPAIAUX
        FROM FCM_ITEMCHECKLIST
        JOIN FCM_ESCOPOSTAREFA
          ON FCM_ESCOPOSTAREFA.NRESCOPOSTAREFA = FCM_ITEMCHECKLIST.NRESCOPOSTAREFA
       WHERE FCM_ITEMCHECKLIST.IDATIVO = 'S'
         AND FCM_ITEMCHECKLIST.NRTAREFA IN ({$parametros})

   UNION ALL 

   SELECT DISTINCT FCM_ITEMCHECKLIST.NRESCOPOSTAREFA||'_'||FCM_ITEMCHECKLIST.NRTAREFA AS CODIGO, FCM_ITEMCHECKLIST.NRTAREFA AS CODIGOAUX,  FCM_TAREFA.NMTAREFA AS NOME, 'Atividade' AS TIPO, TO_CHAR(FCM_ITEMCHECKLIST.NRESCOPOSTAREFA) AS CODIGOPAI, FCM_ITEMCHECKLIST.NRESCOPOSTAREFA AS CODIGOPAIAUX
       FROM FCM_ITEMCHECKLIST
       JOIN FCM_TAREFA
         ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
      WHERE FCM_ITEMCHECKLIST.IDATIVO  = 'S'
        AND FCM_ITEMCHECKLIST.NRTAREFA IN ({$parametros})

   UNION ALL

   SELECT DISTINCT TO_CHAR(FCM_ITEMCHECKLIST.NRCRITERIOTAREFA) AS CODIGO, FCM_ITEMCHECKLIST.NRCRITERIOTAREFA AS CODIGOAUX, FCM_CRITERIOTAREFA.DSCRITERIOTAREFA AS NOME, 'Critério' AS TIPO,  FCM_ITEMCHECKLIST.NRESCOPOSTAREFA||'_'||FCM_ITEMCHECKLIST.NRTAREFA AS CODIGOPAI, FCM_ITEMCHECKLIST.NRTAREFA AS CODIGOPAIAUX 
       FROM  FCM_ITEMCHECKLIST
       JOIN FCM_CRITERIOTAREFA 
         ON FCM_CRITERIOTAREFA.NRCRITERIOTAREFA = FCM_ITEMCHECKLIST.NRCRITERIOTAREFA
      WHERE FCM_ITEMCHECKLIST.IDATIVO  = 'S'
        AND FCM_ITEMCHECKLIST.NRTAREFA IN ({$parametros})";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function salvarChecklist($parametros) {

        $sql = "INSERT INTO FCM_CHECKLIST (NRCHECKLIST, NMCHECKLIST, NRTIPOLOCALCLI, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRCHECKLIST, :NMCHECKLIST, :NRTIPOLOCALCLI, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function salvarItens($parametros) {

        $sql = "INSERT INTO FCM_ATIVIDADECHK (NRATIVIDADECHK, NRATIVIDADE, NRCHECKLIST, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRATIVIDADECHK, :NRATIVIDADE, :NRCHECKLIST, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function verificaExistenciaAtividade($parametros) {

        $sql = "SELECT COUNT(*) CONT
                FROM FCM_ITEMCHECKLIST
                WHERE NRATIVIDADE = :NRATIVIDADE
                AND NRTAREFA = :NRTAREFA";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        $st = $std->fetchAll();
        return $st[0]['CONT'];
    }

    public function salvarAtividade($parametros) {
        $sql = "INSERT INTO FCM_TAREFACHK (NRTAREFACHK, NRTAREFA, NRCHECKLIST, NRATIVIDADECHK, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRTAREFACHK, :NRTAREFA, :NRCHECKLIST, :NRATIVIDADECHK, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }


    public function salvarCriterio($parametros) {
        $sql = "INSERT INTO FCM_CRITERIOTAREFACHK (NRCRITERIOTAREFACHK, NRCRITERIOTAREFA, NRTAREFACHK, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRCRITERIOTAREFACHK, :NRCRITERIOTAREFA, :NRTAREFACHK, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function salvarEscopo($parametros) {
        $sql = "INSERT INTO FCM_ESCOPOSTAREFACHK (NRESCOPOSTAREFACHK, NRCHECKLIST, NRTAREFACHK, NRESCOPOSTAREFA, IDATIVO, NRORG, NRORGINCLUSAO, DTINCLUSAO, CDOPERINCLUSAO)
                VALUES (:NRESCOPOSTAREFACHK, :NRCHECKLIST, :NRTAREFACHK, :NRESCOPOSTAREFA, 'S', :NRORG, :NRORGINCLUSAO, SYSDATE, :CDOPERINCLUSAO)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function getChecklist() {
        
        $sql = "SELECT FCM_CHECKLIST.NRCHECKLIST, FCM_CHECKLIST.NMCHECKLIST, FCM_TIPOLOCALCLI.NMTIPOLOCALCLI
                    FROM FCM_CHECKLIST
                    JOIN FCM_TIPOLOCALCLI
                    ON FCM_TIPOLOCALCLI.NRTIPOLOCALCLI = FCM_CHECKLIST.NRTIPOLOCALCLI";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // FUNÇÕES DA TAB DE ITENS E CRITÉRIOS

    public function getItensChecklist() {
        $sql = "SELECT NRATIVIDADE, NMATIVIDADE, DSCORIDATIVIDADE 
                    FROM FCM_ATIVIDADE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirItemChecklist($parametros) {

        $sql = "INSERT INTO FCM_ATIVIDADE (NRATIVIDADE, NMATIVIDADE, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DSCORIDATIVIDADE)
        VALUES (:NRATIVIDADE, :NMATIVIDADE, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :DSCORIDATIVIDADE)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function getCriteriosChecklist($filtro) {

        $sql = "SELECT * FROM FCM_CRITERIOTAREFA
                    WHERE (NRTAREFA = :NRTAREFA OR :NRTAREFA IS NULL)
                    ORDER BY DSCRITERIOTAREFA";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $filtro);        

    }

    public function getTarefasChecklist($parametros) {

        $sql = "SELECT FCM_ITEMCHECKLIST.*, FCM_TAREFA.NMTAREFA, FCM_CRITERIOTAREFA.DSCRITERIOTAREFA, FCM_ESCOPOSTAREFA.NMESCOPOSTAREFA
                            FROM FCM_ITEMCHECKLIST
                    LEFT JOIN FCM_TAREFA
                            ON FCM_TAREFA.NRTAREFA = FCM_ITEMCHECKLIST.NRTAREFA
                    LEFT JOIN FCM_CRITERIOTAREFA
                            ON FCM_CRITERIOTAREFA.NRCRITERIOTAREFA = FCM_ITEMCHECKLIST.NRCRITERIOTAREFA
                    LEFT JOIN FCM_ESCOPOSTAREFA
                            ON FCM_ESCOPOSTAREFA.NRESCOPOSTAREFA = FCM_ITEMCHECKLIST.NRESCOPOSTAREFA
                        WHERE FCM_ITEMCHECKLIST.NRATIVIDADE = :NRATIVIDADE
                            AND FCM_ITEMCHECKLIST.IDATIVO = 'S'";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);        

    }
    public function inserirTarefaChecklist($parametros) {
        $sql = "INSERT INTO FCM_ITEMCHECKLIST (NRITEMCHECKLIST, NRATIVIDADE, NRTAREFA, NRCRITERIOTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, NRESCOPOSTAREFA)
        VALUES (:NRITEMCHECKLIST, :NRATIVIDADE, :NRTAREFA, :NRCRITERIOTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :NRESCOPOSTAREFA)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function inativarTarefasChecklist($parametros) {

        $sql = "UPDATE FCM_ITEMCHECKLIST 
                SET IDATIVO = 'N',
                    NRORGULTATU = :NRORGULTATU,
                    DTULTATU = SYSDATE, 
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRITEMCHECKLIST = :NRITEMCHECKLIST";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);

    }

    public function updateCorItemChecklist($parametros) {

        $sql = "UPDATE FCM_ATIVIDADE
                    SET DSCORIDATIVIDADE = :DSCORIDATIVIDADE,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU, 
                        NRORGULTATU = :NRORGULTATU
                WHERE NRATIVIDADE = :NRATIVIDADE";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    // FUNÇÕES DA TAB ATIVIDADE E CRITÉRIOS
    public function getAtividadesCriterios() {

        $sql = "SELECT 'T'||NRTAREFA AS CODIGO, NMTAREFA AS NOME, NULL AS CODIGOPAI, 'Atividade' AS TIPO, NRTAREFA AS NR
                    FROM FCM_TAREFA
                    WHERE IDATIVO = 'S'
                UNION ALL
                    SELECT 'C'||NRCRITERIOTAREFA AS CODIGO, DSCRITERIOTAREFA AS NOME, 'T'||NRTAREFA AS CODIGOPAI, 'Critério' AS TIPO, NRCRITERIOTAREFA AS NR
                    FROM FCM_CRITERIOTAREFA
                    WHERE IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirCriterio($parametros) {

        $sql = "INSERT INTO FCM_CRITERIOTAREFA (NRCRITERIOTAREFA, DSCRITERIOTAREFA, NRTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRCRITERIOTAREFA, :DSCRITERIOTAREFA, :NRTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";
    
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function inativarCriterio($parametros) {

        $sql = "UPDATE FCM_CRITERIOTAREFA 
                SET IDATIVO = 'N',
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU
                WHERE 'C'||NRCRITERIOTAREFA = :CODIGO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function updateCriterio($parametros) {

        $sql = "UPDATE FCM_CRITERIOTAREFA
                        SET DSCRITERIOTAREFA = :DSCRITERIOTAREFA,
                            DTULTATU = SYSDATE,
                            CDOPERULTATU = :CDOPERULTATU, 
                            NRORGULTATU = :NRORGULTATU
                    WHERE 'C'||NRCRITERIOTAREFA = :NRCRITERIOTAREFA";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function inserirAtividade($parametros) {

        $sql = "INSERT INTO FCM_TAREFA(NRTAREFA, NMTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRTAREFA, :NMTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function inativarAtividade($parametros) {

        $sql = "UPDATE FCM_TAREFA 
                SET IDATIVO = 'N',
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORGULTATU = :NRORGULTATU
                WHERE 'T'||NRTAREFA = :CODIGO";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function updateAtividade($parametros) {

        $sql = "UPDATE FCM_TAREFA
                        SET NMTAREFA = :NMTAREFA,
                            DTULTATU = SYSDATE,
                            CDOPERULTATU = :CDOPERULTATU, 
                            NRORGULTATU = :NRORGULTATU
                    WHERE 'T'||NRTAREFA = :NRTAREFA";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function verificarExistenciaFilhos($parametros) {
        
        $sql = "SELECT COUNT(*) FILHOS
                FROM FCM_CRITERIOTAREFA
                WHERE 'T'||NRTAREFA = :CODIGO
                AND IDATIVO = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        $st = $std->fetchAll();
        return $st[0]['FILHOS'];
    }

    // FUNÇÕES DA TAB DE ESCOPO
    public function getEscoposServicos() {

        $sql = "SELECT * FROM FCM_ESCOPOSTAREFA
                WHERE IDATIVO = 'S'";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function inserirEscopoServico($parametros) {

        $sql = "INSERT INTO FCM_ESCOPOSTAREFA (NRESCOPOSTAREFA, NMESCOPOSTAREFA, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES (:NRESCOPOSTAREFA, :NMESCOPOSTAREFA, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function inativarEscopos($parametros) {

        $sql = "UPDATE FCM_ESCOPOSTAREFA 
                SET IDATIVO = 'N',
                    NRORGULTATU = :NRORGULTATU,
                    DTULTATU = SYSDATE, 
                    CDOPERULTATU = :CDOPERULTATU
                WHERE NRESCOPOSTAREFA IN (:NRESCOPOSTAREFA)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($parametros);

    }
}