import {
  Button, Date as DateInput, DateRange, Grid, LoadingService, Modal, ModalService, Select, SelectMultiple,
} from '@zeedhi/common';
import {
  dayjs, Http, IDictionary, Metadata,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

// Este contoller é usado também na tela  "Calendário", pelo botão Transferir Tarefas.

export class Fm00004001TransferirTarefasController {
  private PersistentModal!: Modal;

  public async createBasicUsageModal() {
    this.PersistentModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_003_modal_transferir_tarefas' });
  }

  public destroyBasicUsageModal() {
    this.PersistentModal.destroy();
  }

  public openBasicModal() {
    this.PersistentModal.show();
  }

  public closeBasicModal() {
    this.PersistentModal.hide();
  }

  public setTask() {
    const funcionarioOrigem = Metadata.getInstance<Select>('SelectFuncionarioOrigem');
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    grid.datasource.data = [];
    if (funcionarioOrigem.value == null) {
      grid.isVisible = false;
    } else {
      LoadingService.show();
      const dataRange = Metadata.getInstance<DateRange>('dataPeriodo').value || null;
      grid.isVisible = true;
      Http.get(`/taskOfTheEmployee?nr=${funcionarioOrigem.value}&periodo=${dataRange}`).then((response: IDictionary) => {
        response.data.forEach((element: IDictionary) => {
          grid.datasource.data.push({
            id: element.NRCHECKLISTAGEND,
            name: element.NMCHECKLISTAGEND,
            local: element.NMLOCALCLIENTE,
            date: element.DTHORAAGEND,
          });
        });
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  public async transfer() {
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    const dadosHaAtualizar = grid.selectedRows.map((item: any) => item.id);
    let idVistoria = dadosHaAtualizar.filter((value: any) => value.indexOf('V-') >= 0);
    const idTarefa = dadosHaAtualizar.filter((value: any) => value.indexOf('V-') < 0);

    let funcionarioDestino = Metadata.getInstance<Select>('SelectFuncionarioDestino');
    const funcionarioOrigem = Metadata.getInstance<Select>('SelectFuncionarioOrigem').value;

    if (funcionarioDestino.isValid() && (idVistoria.length || idTarefa.length) && (funcionarioDestino.value !== funcionarioOrigem)) {
      funcionarioDestino = funcionarioDestino.value;
      let success = true;

      if (idVistoria.length) {
        idVistoria = idVistoria.map((value: string) => Number(value.substr(2)));

        await Http.post('/transferVistoria', {
          funcionarioDestino,
          idVistoria,
          funcionarioOrigem,
        }).then(() => {
          grid.selectedRows = [];
          this.setTask();
          Utilities.showSuccessMessage();
          success = true;
        }).catch((error: any) => {
          Utilities.showErrorMessage(error);
        });
      }

      if (idTarefa.length && success) {
        await Http.post('/transferTask', {
          funcionarioDestino,
          idTarefa,
          funcionarioOrigem,
        }).then(() => {
          grid.selectedRows = [];
          this.setTask();
          Utilities.showSuccessMessage();
        }).catch((error: any) => {
          Utilities.showErrorMessage(error);
        });
      }
    } else if (!funcionarioDestino.isValid()) {
      Utilities.alertMessage('Selecione um funcionário de destino.', 'warning');
    } else if (funcionarioDestino.value === funcionarioOrigem) {
      Utilities.alertMessage('Selecione um funcionário de destino diferente.', 'warning');
    } else {
      Utilities.alertMessage('Nenhuma Tarefa selecionada.', 'error');
    }
  }

  private dadosOriginalDeTranferencia: any = [];
  /* eslint-disable */
  public historicoTransfer({ component }: { component: Grid }) {
    Http.get('/getHistorico').then((response: IDictionary) => {
      const dadosBanco = response.data.dataset.historico;
      let dadosHaModificar = dadosBanco.map((elemento: any) => elemento.NOME);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_NOME').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      dadosHaModificar = dadosBanco.map((elemento: any) => elemento.FUNCORIGEM);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCORIGEM').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      dadosHaModificar = dadosBanco.map((elemento: any) => elemento.FUNCDESTINO);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCDESTINO').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      dadosHaModificar = dadosBanco.map((elemento: any) => elemento.RESPONSAVEL);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_RESPONSAVEL').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      component.datasource.updateData(dadosBanco);
      this.dadosOriginalDeTranferencia = dadosBanco;
      component.datasource.get();
    });
  }

  public proximaPagina({ component }: any) {
    component.parent.datasource.page += 1;
    if (component.parent.paginationLength === component.parent.datasource.page) {
      component.disabled = true;
    }
    component.parent.datasource.get();
    Metadata.getInstance<Button>(`BotaoPaginaAnterior_${component.name.split('_')[1]}`).disabled = false;
    component.parent.datasource.get();
  }

  public paginaAnterior({ component }: any) {
    if (component.parent.datasource.page > 1) {
      component.parent.datasource.page -= 1;
      if (component.parent.datasource.page === 1) {
        component.disabled = true;
      }
      component.parent.datasource.get();
      Metadata.getInstance<Button>(`BotaoProximaPagina_${component.name.split('_')[1]}`).disabled = false;
    }
    component.parent.datasource.get();
  }

  public async filtrar() {
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    grid.datasource.updateData(this.dadosOriginalDeTranferencia);
    let copiaDatasource: any = grid.datasource;
    copiaDatasource = copiaDatasource.allData || grid.datasource.data;
    const dadosAFiltrarNome = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_NOME').value;
    copiaDatasource = (dadosAFiltrarNome !== '' && dadosAFiltrarNome !== null && dadosAFiltrarNome.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarNome.includes(elemento.NOME)) : copiaDatasource;
    const dadosAFiltrarDataTarefa = Metadata.getInstance<DateInput>('CampoParaFiltrar_GridComTarefas_DTHORAAGEND').value;
    copiaDatasource = (dadosAFiltrarDataTarefa !== '' && dadosAFiltrarDataTarefa !== null && dadosAFiltrarDataTarefa.length === 10) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarDataTarefa === elemento.DTHORAAGEND.split(' ')[0]) : copiaDatasource;
    const dadosAFiltrarFuncOrigem = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCORIGEM').value;
    copiaDatasource = (dadosAFiltrarFuncOrigem !== '' && dadosAFiltrarFuncOrigem !== null && dadosAFiltrarFuncOrigem.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarFuncOrigem.includes(elemento.FUNCORIGEM)) : copiaDatasource;
    const dadosAFiltrarFuncDestino = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCDESTINO').value;
    copiaDatasource = (dadosAFiltrarFuncDestino !== '' && dadosAFiltrarFuncDestino !== null && dadosAFiltrarFuncDestino.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarFuncDestino.includes(elemento.FUNCDESTINO)) : copiaDatasource;
    const dadosAFiltrarDataTransfer = Metadata.getInstance<DateInput>('CampoParaFiltrar_GridComTarefas_DTINCLUSAO').value;
    copiaDatasource = (dadosAFiltrarDataTransfer !== '' && dadosAFiltrarDataTransfer !== null && dadosAFiltrarDataTransfer.length === 10) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarDataTransfer === elemento.DTINCLUSAO.split(' ')[0]) : copiaDatasource;
    const dadosAFiltrarResponsavel = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_RESPONSAVEL').value;
    copiaDatasource = (dadosAFiltrarResponsavel !== '' && dadosAFiltrarResponsavel !== null && dadosAFiltrarResponsavel.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarResponsavel.includes(elemento.RESPONSAVEL)) : copiaDatasource;
    grid.datasource.updateData(copiaDatasource);
    grid.datasource.get();
  }

  public sortFunction(value1: any, value2: any): boolean {
    return (dayjs(value1, 'DD/MM/YYYY H:m').isAfter(dayjs(value2, 'DD/MM/YYYY H:m')));
  }
}
